###
### Copyright 2002 Ximian, Inc.
###
### This program is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License, version 2,
### as published by the Free Software Foundation.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
###

import sys
import os
import os.path
import glob
from ConfigParser import ConfigParser
import string
from yum.packages import YumInstalledPackage, YumLocalPackage
import rumtalk
import rumformat
import rumcommand
import rumpackagecmds

class TransactCmd(rumpackagecmds.PackageCmd):
    def local_opt_table(self):
        return [["N",  "dry-run", "", "Perform a dry run"]]

class UpdateCmd(TransactCmd):

    def name(self):
        return "update"

    def aliases(self):
        return ["up"]

    def is_basic(self):
        return 1

    def category(self):
        return "package"

    def arguments(self):
        return "<repo> [repo] ..."

    def description_short(self):
        return "Perform an update"

    def local_opt_table(self):
        return TransactCmd.local_opt_table(self) + [["d",  "download-only", "", "Only download packages"]]

    def execute(self, options_dict, non_option_args):
        yum = self.yum()

        if len(non_option_args) > 0:
            updates = []
            for repo in non_option_args:
                updates += self.get_updates(repo)
        else:
            updates = self.get_updates()

        for (new_pkg, old_pkg) in updates:
            yum.tsInfo.addUpdate(new_pkg, old_pkg)

        self.start_transaction(options_dict.has_key('dry-run'))

rumcommand.register(UpdateCmd)


class InstallCmd(TransactCmd):

    def name(self):
        return "install"

    def aliases(self):
        return ["in"]

    def is_basic(self):
        return 1

    def category(self):
        return "package"

    def arguments(self):
        return "<package> ..."

    def description_short(self):
        return "Perform an install"

    def local_opt_table(self):
        return TransactCmd.local_opt_table(self) + [["d",  "download-only", "", "Only download packages"]]

    def separate_args(self, args):
        installs = []
        removals = []

        for arg in args:
            if arg.startswith('~'):
                removals.append(arg[1:])
            else:
                installs.append(arg)

        return installs, removals

    def execute(self, options_dict, non_option_args):
        if len(non_option_args) < 1:
            self.usage()
            return 1

        yum = self.yum()

        for arg in non_option_args:
            if os.path.exists(arg):
                p = YumLocalPackage(ts=yum.read_ts, filename=arg)
                if yum.rpmdb.installed(po=p):
                    rumtalk.error("%s-%s is already installed" % (p.name, rumformat.naevr_to_str(p.pkgtup)))
                    return False
                
                if p:
                    yum.install(p)
                else:
                    rumtalk.error("'%s' doesn't seem to be a valid package." % arg)
            else:
                install_args, remove_args = self.separate_args(non_option_args)
                installs, updates = self.find_available_packages(install_args)
                
                exactmatches, matches, unmatched = self.find_packages(remove_args, installed=True)
                if len(remove_args) > 0 and len(unmatched) > 0:
                    rumtalk.error("Could not find package '%s'" % unmatched[0])
                    return False

                removals = exactmatches + matches

                for install in installs:
                    yum.install(install)

                for update in updates:
                    exactmatches, matches, unmatched = self.find_packages([update.name], installed=True)
                    yum.tsInfo.addUpdate(update, exactmatches[0])

                for removal in removals:
                    yum.remove(removal)

        self.start_transaction(dryrun=options_dict.has_key('dry-run'),
                               download_only=options_dict.has_key('download-only'))
                    
                
rumcommand.register(InstallCmd)


class RemoveCmd(TransactCmd):

    def name(self):
        return "remove"

    def aliases(self):
        return ["rm"]

    def is_basic(self):
        return 1

    def category(self):
        return "package"

    def arguments(self):
        return "<package> ..."

    def description_short(self):
        return "Perform a removal"

    def execute(self, options_dict, non_option_args):
        if len(non_option_args) < 1:
            self.usage()
            return 1

        yum = self.yum()

        exactmatches, matches, unmatched = self.find_packages(non_option_args, installed=1)
        if len(unmatched) > 0:
            rumtalk.error("Could not find package '%s'" % unmatched[0])
            return False

        plist = exactmatches + matches

        for p in plist:
            yum.remove(p)

        self.start_transaction(dryrun=options_dict.has_key('dry-run'))

rumcommand.register(RemoveCmd)
